#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#include <string.h>

#define LINE_LENGTH 256
#define FILE_NAME 256

static void error_fatal (char *message);

/* Program demonstrira funkcije za kreiranje privremenih fajlova. */
int main (int argc, char **argv){
  char name[FILE_NAME];		/* Ime privremenog fajla. */
  char line[LINE_LENGTH];	/* Linija teksta koja se
				 * upisuje u privremeni fajl. */
  FILE *file;			/* Objekat koji opisuje fajl. */
  int fd;

  /* Kreira se i ispisuje jedno ime privremenog fajla. 
     Funkcija mkstemp kreira fajl pri cemu zadnjih 6 karaktera stringa moraju biti XXXXXX.
     Ove karaktere zamenjuje stringom tako da ime fajla bude jedinstveno.
     Fajl se kreira sa pravima 0600 i otvoreni fajl deskriptor se vraca kao povratna vrednost. */
  strcpy (name, "tmp.XXXXXX");
  if ( (fd = mkstemp (name)) < 0)
    error_fatal ("mkstemp error");
  if (printf ("%s\n", name) < 0)
    error_fatal ("printf error");
  if (close (fd) < 0)
    error_fatal ("close error");

  /* Kreira se privremeni fajl. */
  if ((file = tmpfile ()) == NULL)
    error_fatal ("tmpfile error");

  /* Upisuje se tekst u ovaj fajl. */
  if (fputs ("Hello, world!\n", file) == EOF)
    error_fatal ("fputs error");

  /* Ucitava se natrag i stampa na standardni izlaz tekst iz
   * privremenog fajla.*/
  rewind (file);
  if (fgets (line, LINE_LENGTH, file) == NULL)
    error_fatal ("2nd fgets error");
  if (fputs (line, stdout) == EOF)
    error_fatal ("2nd fputs error");

  /* Zavrsava se program. */
  exit (EXIT_SUCCESS);
}

static void error_fatal (char *message){
  perror (message);
  exit (EXIT_FAILURE);
}
